<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
namespace ServicenowModule\Controller;

use App\Lib\Exceptions\MissingDbBackendException;
use App\Model\Table\ProxiesTable;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\ORM\TableRegistry;
use GuzzleHttp\Client;
use itnovum\openITCOCKPIT\Core\DbBackend;
use itnovum\openITCOCKPIT\Core\System\Gearman;
use itnovum\openITCOCKPIT\Exceptions\HostNotFoundException;
use itnovum\openITCOCKPIT\Exceptions\ServiceNotFoundException;
use NotFoundException;
use ServicenowModule\Lib\ServicenowData;
use ServicenowModule\Model\Table\ServicenowIncidentsTable;
use ServicenowModule\Model\Table\ServicenowSettingsTable;

class ServicenowAcknowledgeController extends AppController {

    private $servicenowStateList = [
        1 => 'New',
        2 => 'In Progress',
        3 => 'On Hold',
        6 => 'Resolved',
        7 => 'Closed',
        8 => 'Canceled'
    ];

    private $proxySettings;
    private $servicenowSettings;

    /**
     * @throws MissingDbBackendException
     * @throws HostNotFoundException
     * @throws ServiceNotFoundException
     */
    public function submit() {
        $this->DbBackend = new DbBackend();
        /** @var ServicenowSettingsTable $ServicenowSettingsTable */
        $ServicenowSettingsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowSettings');
        /** @var ServicenowIncidentsTable $ServicenowIncidentsTable */
        $ServicenowIncidentsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowIncidents');
        /** @var ProxiesTable $ProxiesTable */
        $ProxiesTable = TableRegistry::getTableLocator()->get('Proxies');
        /** @var ServicenowData $ServicenowData */
        $ServicenowData = new ServicenowData();
        /** @var Gearman $GearmanClient */
        $GearmanClient = new Gearman();
        /** @var Client $guzzleclient */
        $guzzleclient = new Client();


        $this->proxySettings = $ProxiesTable->getSettings();
        $this->servicenowSettings = $ServicenowSettingsTable->find()->first()->toArray();

        if (!$this->request->is('post') || $this->servicenowSettings['two_way'] != 1) {
            throw new MethodNotAllowedException();
        }

        $this->autoRender = false;
        $incident_id = $this->request->getData('incident_id');
        $state_number = intval($this->request->getData('state'));
        $state = $this->servicenowStateList[$state_number];
        $resolution_notes = $this->request->getData('close_notes');
        $resolution_code = $this->request->getData('close_code');
        $sys_resolved_by = $this->request->getData('sys_resolved_by');
        $updated_by = $this->request->getData('updated_by');

        $incidentEntity = $ServicenowIncidentsTable->find()
            ->where([
                'ServicenowIncidents.incident_id' => $incident_id
            ])
            ->first();

        if (empty($incidentEntity)) {
            throw new \Cake\Http\Exception\NotFoundException();
        }
        $incident = $incidentEntity->toArray();

        if ($sys_resolved_by != null && !empty($sys_resolved_by)) {
            $options = [
                'auth'    => [$this->servicenowSettings['user_id'], $this->servicenowSettings['user_password']],
                'headers' => [
                    'Accept'       => 'application/json',
                    'Content-Type' => 'application/json'
                ]
            ];
            if ($this->servicenowSettings['use_proxy']) {
                $options['proxy'] = [
                    'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                    'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
                ];
            } else {
                $options['proxy'] = [
                    'http'  => false,
                    'https' => false
                ];
            }
            $serviceNowUrl = sprintf('%s/api/now/table/sys_user', $this->servicenowSettings['domain']);
            $response = $guzzleclient->get($serviceNowUrl . '/' . $sys_resolved_by, $options);
            $result = json_decode($response->getBody()->getContents(), true)['result'];
            $resolved_by_name = $result['name'];
        }

        $type = '';
        if (!empty($incident['host_uuid'])) {
            $type = 'Host';
            if (!empty($incident['service_uuid'])) {
                $type = 'Service';
            }
        }


        $ack_message = sprintf(
            'ServiceNow Incident set to [b]%s[/b] by [b]%s[/b] due to ServiceNow.',
            $state,
            isset($resolved_by_name) ? $resolved_by_name : $updated_by
        );

        if ($state_number == 6 || $state_number == 7) {   // 6 = resolved; 7 = closed;
            $incident['resolution_code'] = $resolution_code;
            $incident['resolution_notes'] = $resolution_notes;
        }

        $incidentURL = sprintf('%s/nav_to.do?uri=/incident.do?sys_id=%s', $this->servicenowSettings['url'], $incident['sys_id']);
        $ack_message .= sprintf('\nServiceNow Ticket: [url=%s]%s[/url]', $incidentURL, $incident_id);

        if ($type == "Host") {
            $host = $ServicenowData->getHost($incident['host_uuid']);

            $GearmanClient->sendBackground('cmd_external_command', [
                'command'     => 'ACKNOWLEDGE_HOST_PROBLEM',
                'parameters'  => [
                    'hostUuid'   => $host['uuid'],
                    'sticky'     => 2, // 2 = sticky with Nagios Core, 1 = Sticky with Naemon Core, 2 works for both :)
                    'notify'     => 0, // do not enable - loop!
                    'persistent' => 1,
                    'author'     => $updated_by,
                    'comment'    => $ack_message,
                ],
                'satelliteId' => null
            ]);

        }

        if ($type == "Service") {
            $service = $ServicenowData->getService($incident['service_uuid']);

            $GearmanClient->sendBackground('cmd_external_command', [
                'command'     => 'ACKNOWLEDGE_SVC_PROBLEM',
                'parameters'  => [
                    'hostUuid'    => $incident['host_uuid'],
                    'serviceUuid' => $service['uuid'],
                    'sticky'      => 2, // 2 = sticky with Nagios Core, 1 = Sticky with Naemon Core, 2 works for both :)
                    'notify'      => 0, // do not enable - loop!
                    'persistent'  => 1,
                    'author'      => $updated_by,
                    'comment'     => $ack_message,
                ],
                'satelliteId' => null
            ]);
        }

        $incident['state'] = $state_number;
        $incidentEntity = $ServicenowIncidentsTable->patchEntity($incidentEntity, $incident);
        $ServicenowIncidentsTable->save($incidentEntity);
        if ($incidentEntity->hasErrors()) {
            $this->serializeCake4ErrorMessage($incidentEntity);
        }
        return;
    }
}
