<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace ServicenowModule\Controller;

use Cake\ORM\TableRegistry;
use ServicenowModule\Model\Table\ServicenowHostspecificSettingsTable;
use ServicenowModule\Model\Table\ServicenowIncidentsTable;
use ServicenowModule\Model\Table\ServicenowServicespecificSettingsTable;
use ServicenowModule\Model\Table\ServicenowSettingsTable;

/**
 * Class ServicenowSettingsController
 * @package ServicenowModule\Controller
 */
class ServicenowSettingsController extends AppController {

    private $servicenowStateList = [
        1 => 'New',
        2 => 'In Progress',
        3 => 'On Hold',
        6 => 'Resolved',
        7 => 'Closed',
        8 => 'Canceled'
    ];

    public function index() {
        if (!$this->isAngularJsRequest()) {
            //Only ship html template
            return;
        }

        /** @var ServicenowSettingsTable $ServicenowSettingsTable */
        $ServicenowSettingsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowSettings');
        $servicenowSettings = $ServicenowSettingsTable->getServicenowSettings();

        if ($this->request->is('get')) {
            $this->set('servicenowSettings', $servicenowSettings);
            $this->viewBuilder()->setOption('serialize', [
                'servicenowSettings'
            ]);
            return;
        }

        //Save POST||PUT Request
        if ($this->request->is('post') || $this->request->is('put')) {
            $data = $this->request->getData();

            $enity = $ServicenowSettingsTable->getServicenowSettingsEntity();
            $enity = $ServicenowSettingsTable->patchEntity($enity, $data);

            $ServicenowSettingsTable->save($enity);
            if ($enity->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $enity->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            $this->set('servicenowSettings', $enity);
            $this->viewBuilder()->setOption('serialize', ['servicenowSettings']);
        }
    }

    /**
     * @param null $uuid
     */
    public function hostspecific($uuid = null) {
        if (!$this->isAngularJsRequest()) {
            //Only ship html template
            return;
        }

        /** @var ServicenowSettingsTable $ServicenowSettingsTable */
        $ServicenowSettingsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowSettings');
        /** @var ServicenowHostspecificSettingsTable $ServicenowHostspecificSettingsTable */
        $ServicenowHostspecificSettingsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowHostspecificSettings');
        /** @var ServicenowIncidentsTable $ServicenowIncidentsTable */
        $ServicenowIncidentsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowIncidents');

        if ($this->request->is('post') || $this->request->is('put')) {
            $data = $this->request->getData('ServicenowHostspecificSettings');
            $data['modified'] = date("Y-m-d H:i:s");

            if (isset($data['id'])) {
                $settingsEntity = $ServicenowHostspecificSettingsTable->get(intval($data['id']));
            } else {
                $settingsEntity = $ServicenowHostspecificSettingsTable->newEmptyEntity();
            }

            $settingsEntity = $ServicenowHostspecificSettingsTable->patchEntity($settingsEntity, $data);
            $ServicenowHostspecificSettingsTable->save($settingsEntity);

            if (!$settingsEntity->hasErrors()) {
                $this->set('id', $settingsEntity->id);
                $this->viewBuilder()->setOption('serialize', ['id']);
                return;
            }
            $this->serializeCake4ErrorMessage($settingsEntity);
            return;
        }

        if ($uuid !== null) {
            $settingsEntity = $ServicenowHostspecificSettingsTable->getServicenowHostspecificSettingsByHostuuid($uuid);

            $incidentEntity = $ServicenowIncidentsTable->find()
                ->where([
                    'ServicenowIncidents.host_uuid'    => $uuid,
                    'ServicenowIncidents.service_uuid' => ''
                ])
                ->first();

            if (empty($incidentEntity)) {
                $incident = null;
            } else {
                $incident = $incidentEntity->toArray();
            }

            $ServicenowSettings = $ServicenowSettingsTable->getServicenowSettings();

            $globalsettings = ['ServicenowSettings' => $ServicenowSettings];
            $settings = ['ServicenowHostspecificSettings' => $settingsEntity];
            $incident = ['ServicenowIncident' => $incident];

            if (isset($globalsettings['ServicenowSettings']) && $globalsettings['ServicenowSettings']['impact'] != null && $settings['ServicenowHostspecificSettings']['impact'] == null) {
                $settings['ServicenowHostspecificSettings']['impact'] = $globalsettings['ServicenowSettings']['impact'];
            }
            if (isset($globalsettings['ServicenowSettings']) && $globalsettings['ServicenowSettings']['urgency'] != null && $settings['ServicenowHostspecificSettings']['urgency'] == null) {
                $settings['ServicenowHostspecificSettings']['urgency'] = $globalsettings['ServicenowSettings']['urgency'];
            }
            if (isset($incident['ServicenowIncident']) && isset($incident['ServicenowIncident']['state']) && $incident['ServicenowIncident']['state'] != null) {
                $incident['ServicenowIncident']['state'] = $this->servicenowStateList[$incident['ServicenowIncident']['state']];
            }

            $this->set('globalsettings', $globalsettings);
            $this->set('settings', $settings);
            $this->set('incident', $incident);
            $this->viewBuilder()->setOption('serialize', ['settings', 'incident', 'globalsettings']);
        }
    }

    /**
     * @param null $uuid
     */
    public function servicespecific($uuid = null) {
        if (!$this->isAngularJsRequest()) {
            //Only ship html template
            return;
        }

        /** @var ServicenowSettingsTable $ServicenowSettingsTable */
        $ServicenowSettingsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowSettings');
        /** @var ServicenowServicespecificSettingsTable $ServicenowServicespecificSettingsTable */
        $ServicenowServicespecificSettingsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowServicespecificSettings');
        /** @var ServicenowIncidentsTable $ServicenowIncidentsTable */
        $ServicenowIncidentsTable = TableRegistry::getTableLocator()->get('ServicenowModule.ServicenowIncidents');

        if ($this->request->is('post') || $this->request->is('put')) {
            $data = $this->request->getData('ServicenowServicespecificSettings');
            $data['modified'] = date("Y-m-d H:i:s");

            if (isset($data['id'])) {
                $settingsEntity = $ServicenowServicespecificSettingsTable->get(intval($data['id']));
            } else {
                $settingsEntity = $ServicenowServicespecificSettingsTable->newEmptyEntity();
            }

            $settingsEntity = $ServicenowServicespecificSettingsTable->patchEntity($settingsEntity, $data);
            $ServicenowServicespecificSettingsTable->save($settingsEntity);

            if (!$settingsEntity->hasErrors()) {
                $this->set('id', $settingsEntity->id);
                $this->viewBuilder()->setOption('serialize', ['id']);
                return;
            }
            $this->serializeCake4ErrorMessage($settingsEntity);
            return;
        }

        if ($uuid !== null) {
            $settingsEntity = $ServicenowServicespecificSettingsTable->getServicenowServicespecificSettingsByServiceuuid($uuid);

            $incidentEntity = $ServicenowIncidentsTable->find()
                ->where([
                    'ServicenowIncidents.service_uuid' => $uuid
                ])
                ->first();

            if (empty($incidentEntity)) {
                $incident = null;
            } else {
                $incident = $incidentEntity->toArray();
            }

            $ServicenowSettings = $ServicenowSettingsTable->getServicenowSettings();

            $globalsettings = ['ServicenowSettings' => $ServicenowSettings];
            $settings = ['ServicenowServicespecificSettings' => $settingsEntity];
            $incident = ['ServicenowIncident' => $incident];

            if (isset($globalsettings['ServicenowSettings']) && $globalsettings['ServicenowSettings']['impact'] != null && $settings['ServicenowServicespecificSettings']['impact'] == null) {
                $settings['ServicenowServicespecificSettings']['impact'] = $globalsettings['ServicenowSettings']['impact'];
            }
            if (isset($globalsettings['ServicenowSettings']) && $globalsettings['ServicenowSettings']['urgency'] != null && $settings['ServicenowServicespecificSettings']['urgency'] == null) {
                $settings['ServicenowServicespecificSettings']['urgency'] = $globalsettings['ServicenowSettings']['urgency'];
            }

            if (isset($incident['ServicenowIncident']) && isset($incident['ServicenowIncident']['state']) && $incident['ServicenowIncident']['state'] != null) {
                $incident['ServicenowIncident']['state'] = $this->servicenowStateList[$incident['ServicenowIncident']['state']];
            }

            $this->set('globalsettings', $globalsettings);
            $this->set('settings', $settings);
            $this->set('incident', $incident);
            $this->viewBuilder()->setOption('serialize', ['settings', 'incident', 'globalsettings']);
        }
    }

}
