<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace ServicenowModule\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use ServicenowModule\Model\Entity\ServicenowHostspecificSetting;

/**
 * ServicenowHostspecificSettings Model
 *
 * @method ServicenowHostspecificSetting get($primaryKey, $options = [])
 * @method ServicenowHostspecificSetting newEntity($data = null, array $options = [])
 * @method ServicenowHostspecificSetting[] newEntities(array $data, array $options = [])
 * @method ServicenowHostspecificSetting|false save(EntityInterface $entity, $options = [])
 * @method ServicenowHostspecificSetting saveOrFail(EntityInterface $entity, $options = [])
 * @method ServicenowHostspecificSetting patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method ServicenowHostspecificSetting[] patchEntities($entities, array $data, array $options = [])
 * @method ServicenowHostspecificSetting findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin TimestampBehavior
 */
class ServicenowHostspecificSettingsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('servicenow_hostspecific_settings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->nonNegativeInteger('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('host_uuid')
            ->maxLength('host_uuid', 255)
            ->allowEmptyString('host_uuid');

        $validator
            ->scalar('assignment_group')
            ->maxLength('assignment_group', 255)
            ->allowEmptyString('assignment_group');

        $validator
            ->scalar('assigned_to')
            ->maxLength('assigned_to', 255)
            ->allowEmptyString('assigned_to');

        $validator
            ->scalar('business_service')
            ->maxLength('business_service', 255)
            ->allowEmptyString('business_service');

        $validator
            ->allowEmptyString('impact');

        $validator
            ->allowEmptyString('urgency');

        return $validator;
    }

    /**
     * @param string $uuid
     * @return array|EntityInterface|ServicenowHostspecificSetting|null
     */
    public function getServicenowHostspecificSettingsByHostuuid(string $uuid) {
        $default = [
            'host_uuid'        => null,
            'assignment_group' => null,
            'assigned_to'      => null,
            'business_service' => null,
            'impact'           => null,
            'urgency'          => null,
        ];

        $result = $this->find()
            ->where([
                'ServicenowHostspecificSettings.host_uuid' => $uuid
            ])
            ->first();

        if (empty($result)) {
            return $this->newEntity($default);
        }

        return $result;
    }
}
