<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace ServicenowModule\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use ServicenowModule\Model\Entity\ServicenowIncident;

/**
 * ServicenowIncidents Model
 *
 * @method ServicenowIncident get($primaryKey, $options = [])
 * @method ServicenowIncident newEntity($data = null, array $options = [])
 * @method ServicenowIncident[] newEntities(array $data, array $options = [])
 * @method ServicenowIncident|false save(EntityInterface $entity, $options = [])
 * @method ServicenowIncident saveOrFail(EntityInterface $entity, $options = [])
 * @method ServicenowIncident patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method ServicenowIncident[] patchEntities($entities, array $data, array $options = [])
 * @method ServicenowIncident findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin TimestampBehavior
 */
class ServicenowIncidentsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('servicenow_incidents');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->nonNegativeInteger('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->allowEmptyString('state');

        $validator
            ->scalar('resolution_code')
            ->maxLength('resolution_code', 255)
            ->allowEmptyString('resolution_code');

        $validator
            ->scalar('resolution_notes')
            ->maxLength('resolution_notes', 255)
            ->allowEmptyString('resolution_notes');

        $validator
            ->scalar('host_uuid')
            ->maxLength('host_uuid', 255)
            ->allowEmptyString('host_uuid');

        $validator
            ->scalar('service_uuid')
            ->maxLength('service_uuid', 255)
            ->allowEmptyString('service_uuid');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        return $rules;
    }
}
